% SUM-MAX MAPS FOR TEMPLATE MATCHING INFERENCE
%% load in learned template, set parameter values
clear; close all; 
load 'learnedTemplate'; % load in the learned template
mex Cfind.c;   % compile C code for template matching inference
%% load in test image, generate multiple resolutions
numResolution = 8;  % total number of resolutions to search 
allSizex = zeros(1, numResolution); allSizey = zeros(1, numResolution); 
tmpIm = imread('test.jpg'); 
if size(tmpIm,3) == 3
     tmpIm = rgb2gray(tmpIm);
end
J0 = single(tmpIm); 
MAX1maps = cell(numResolution, numOrient); 
SUM2map = cell(1, numResolution); 
imgResolution = .30 + ((1:numResolution)-1)*.1; 
for(img=1:numResolution)
    J{img} = imresize(J0, imgResolution(img), 'nearest');  % scaled images
    [sizex, sizey] = size(J{img}); 
    allSizex(img) = sizex; allSizey(img) = sizey; % sizes of images at multiple resolutions
    sizexSubsample = floor(sizex/subsample); sizeySubsample = floor(sizey/subsample); 
    SUM2map{img} = single(zeros(sizexSubsample, sizeySubsample));  % log-likelihood score maps for testing images
    for (orient=1:numOrient)
        MAX1map{img, orient} = single(zeros(sizexSubsample, sizeySubsample)); 
    end
    translatedTemplate{img} = single(zeros(sizex, sizey));  % symbolic sketch on testing image
end
%% compute SUM1 maps for all resolutions
disp(['start filtering']); tic
SUM1map = ApplyFilterfft(J, allFilter, localOrNot, localHalfx, localHalfy, thresholdFactor);   % filter testing image at all resolutions
disp(['filtering time: ' num2str(toc) ' seconds']);
MAX2score = single(zeros(1, numResolution));  % maximum log-likelihood score at each resolution
%% detect the object and superpose template
disp(['start finding']); tic
Cfind(numResolution, allSizex, allSizey, numOrient, ...
      locationShiftLimit, orientShiftLimit, subsample, saturation,  ...
      halfFilterSize, SUM1map, allSymbol(1, :), ...
      numElement, selectedOrient, selectedx, selectedy, selectedlambda, selectedLogZ, ...
      MAX1map, SUM2map, translatedTemplate, MAX2score);
disp(['mex-C finding time: ' num2str(toc) ' seconds']);
%% write the html code for reproducibility page
[maxOverResolution ind] = max(MAX2score);   % most likely resolution
pic = imresize(imread('test.jpg'), imgResolution(ind), 'nearest'); showImage; 
saveas(gcf, [outputFolder 'eps/inputTest.eps'], 'epsc');
saveas(gcf, [outputFolder 'png/inputTest.png'], 'png');
pic = J{ind}+translatedTemplate{ind}*100.; showImage; 
saveas(gcf, [outputFolder 'eps/outputTest.eps'], 'eps');
saveas(gcf, [outputFolder 'png/outputTest.png'], 'png');
%% write the html code for reproducibility page
heightstr = '"height=150> '; 
disp(['<IMG SRC="' outputFolder 'png/inputTest' '.png' heightstr]);
disp(['<IMG SRC="' outputFolder 'png/outputTest' '.png' heightstr]);
%% write the LaTex code for paper 
heightstr = 'height=.9in}'; 
disp(['\psfig{figure=' outputFolder 'eps/inputTest' '.eps,'  heightstr]); 
disp(['\psfig{figure=' outputFolder 'eps/outputTest' '.eps,'  heightstr]);




